///////    Rubber.java    ///////

/**
 * Clase de creacion del elástico.
 * @author Martín Cortés, Mauricio Ludueña
 */
public class Rubber extends PhysicsElement  {
    //Atributos privados no aparecen en javadoc
   private static int id=0;  // Rubber identification
   /**
    * Largo en reposo.
    */
   protected final float restLength;
   private  final float stiffness;
  /**
    * Extremos a y b del elástico.
    */
   protected AttachableElement a_end, b_end;
   //El metodo privado no aparece en el javadoc
   private Rubber(){   // nobody can create a block without state
      super(id++);
      stiffness = restLength = 0;
   }
   /**
    * Contructor.   Crea un nuevo elástico sin extremos.
    * @param restLength  Largo en reposo.
    * @param stiffness   Viscosidad.
    */
   public Rubber(float restLength, float stiffness){
      super(id++);
      this.restLength = restLength;
      this.stiffness = stiffness;
      a_end = b_end = null;
   }
   /**
    * Adhiere el extremo A a un Elemento.
    * @param ae Elemento al cual se adhiere el extremo.
    */
   public void attachAend (AttachableElement ae) {
      a_end = ae;
      ae.attachRubber(this);
   }
   /**
    * Separa el extremo A del elástico.
    */
   public void detachAend(){
      if (a_end == null) return;
      a_end.detachRubber(this);
      a_end=null;
   }
   
   /**
    * Adhiere el extremo B a un Elemento.
    * @param ae Elemento al cual se adhiere el extremo.
    */
   public void attachBend (AttachableElement ae) {
      b_end = ae;
      ae.attachRubber(this);
   }
   /**
    * Separa el extremo B del elástico.
    */
   public void detachBend(){
      if (b_end == null) return;
      b_end.detachRubber(this);
      b_end = null;
   }
   /**
    * Entrega la posicion del extremo A del elástico.
    * @return El vector posicion de A.
    */
   public Vector2D getAendPosition() {
      if (a_end != null)
         return a_end.getPosition();
      if (b_end != null)
         return b_end.getPosition().minus(new Vector2D(0,restLength));
      return new Vector2D();
   }
   
   /**
    * Entrega la posicion B del elástico.
    * @return El vector posicion de A.
    */
   public Vector2D getBendPosition() {
      if (b_end != null)
         return b_end.getPosition();
      if (a_end != null)
         return a_end.getPosition().plus(new Vector2D(0,restLength));
      return new Vector2D(0,restLength);
   }
   /**
    * Entrega el vector del largo del elástico.
    * @return Vector largo del elástico.
    */
   protected Vector2D getVector() {
      return getBendPosition().minus(getAendPosition());
   }
   /**
    * Calcula la fuerza del elástico.
    * @param ae elemento al cual se le aplica la fuerza.
    * @return Vector fuerza del elástico.
    */
   public Vector2D getForce(AttachableElement ae) {
      Vector2D force2 = new Vector2D();
      if ((a_end == null) || (b_end == null))
         return force2;
      if ((ae != a_end) && (ae != b_end))
         return force2;
      Vector2D rubberVector = getVector();
		double stretch = rubberVector.module() - restLength;
      
		
		if(stretch>=0)
			force2 = rubberVector.unitary().times(stretch*stiffness);
      else
		   force2 = rubberVector.unitary().times(0);
						
		
		if (ae == a_end) 
		return force2;
      return force2.times(-1.0);
   }
   /**
    * Obtiene la descripcion del elástico.
    * @return El String de la informacion del elástico.
    */
	public String getDescription() {
      return "Rubber_"+ getId()+": "+ Vector2D.getDescription()+" ,"+ Vector2D.getDescription();
   }
        /**
         * Obtiene el estado del elástico, considerando sus 2 extremos.
         * @return El String de estado del elástico.
         */
   public String getState() {
      String r = getAendPosition() + "," + getBendPosition();
      return r;
   }
}
